#include <avr/io.h>
#include <util/delay.h>
#include "touch.h"
#include "utility/mcufriend_shield.h"

#define XM_PORT PORTD
#define XM_PIN PD5
#define XP_PORT PORTA
#define XP_PIN PA7
#define YM_PORT PORTA
#define YM_PIN PA6
#define YP_PORT PORTD
#define YP_PIN PD6

#define X_OFFSET 20
#define Y_OFFSET 20
#define X_GAIN 0.3f
#define Y_GAIN 0.4f
#define Z_TRESHOLD 200

uint16_t readX(void) {
	//Y - wejcia
	PIN_INPUT(YM_PORT, YM_PIN);
	PIN_INPUT(YP_PORT, YP_PIN);
	//Y - bez podcignicia
	PIN_LOW(YM_PORT, YM_PIN);
	PIN_LOW(YP_PORT, YP_PIN);
	//X - wyjcia
	PIN_OUTPUT(XM_PORT, XM_PIN);
	PIN_OUTPUT(XP_PORT, XP_PIN);
	PIN_LOW(XM_PORT, XM_PIN);
	PIN_HIGH(XP_PORT, XP_PIN);
	return getADCValue(YM_PIN) * X_GAIN - X_OFFSET;
}

uint16_t readY(void) {
	//X - wejcia
	PIN_INPUT(XM_PORT, XM_PIN);
	PIN_INPUT(XP_PORT, XP_PIN);
	//X - bez podcignicia
	PIN_LOW(XM_PORT, XM_PIN);
	PIN_LOW(XP_PORT, XP_PIN);
	//Y - wyjcia
	PIN_OUTPUT(YM_PORT, YM_PIN);
	PIN_OUTPUT(YP_PORT, YP_PIN);
	PIN_LOW(YM_PORT, YM_PIN);
	PIN_HIGH(YP_PORT, YP_PIN);
	return getADCValue(XP_PIN) * Y_GAIN - Y_OFFSET;
}

uint16_t readZ(void) {
	//X+, Y- - wejcia
	PIN_INPUT(XP_PORT, XP_PIN);
	PIN_INPUT(YM_PORT, YM_PIN);
	PIN_LOW(XP_PORT, XP_PIN);
	PIN_LOW(YM_PORT, YM_PIN);
	//X-, Y+ - wyjcia
	PIN_OUTPUT(XM_PORT, XM_PIN);
	PIN_OUTPUT(YP_PORT, YP_PIN);
	PIN_LOW(XM_PORT, XM_PIN);
	PIN_HIGH(YP_PORT, YP_PIN);
	uint16_t z1 = getADCValue(XP_PIN) * Y_GAIN - Y_OFFSET;
	uint16_t z2 = getADCValue(YM_PIN) * X_GAIN - X_OFFSET;
	return z2 - z1;
}

uint8_t readTouch(uint16_t* x, uint16_t* y) {
	if (readZ() > Z_TRESHOLD) {
		restorePins();
		return 0;
	} else {
		uint32_t _x = 0;
		uint32_t _y = 0;
		for (uint8_t i = 0; i < 10; i++) {
			_x += readX();
			_y += readY();
		}
		*x = _x / 10;
		*y = _y / 10;
		restorePins();
		return 1;
	}
}

uint16_t getADCValue(uint8_t pin) {
	ADMUX = pin;
	ADCSRA |= _BV(ADEN); //wcz ADC
	ADMUX |= _BV(REFS0); //AVCC
	//preskaler 128
	ADCSRA |= _BV(ADPS0)|_BV(ADPS1)|_BV(ADPS2);
	//rozpocznij konwersj
	ADCSRA |= _BV(ADSC);
	//zaczekaj na wynik
	while(ADCSRA & _BV(ADSC));
	//pobierz wynik
	return ADCW;
}

void restorePins(void) {
	PIN_OUTPUT(XM_PORT, XM_PIN);
	PIN_OUTPUT(XP_PORT, XP_PIN);
	PIN_OUTPUT(YM_PORT, YM_PIN);
	PIN_OUTPUT(YP_PORT, YP_PIN);
}